/*
	Find (print) all 0/1-vectors in the affine span of r given vectors in R^n.
*/
#include <algorithm>
#include <fstream>
#include <iomanip>
#include <iostream> 
#include <cstdlib>
#include <sstream>
#include <cassert>
#include <vector>
#include <queue>
#include <map>
#include <set>
#define FI(x) (x).first
#define SE(x) (x).second
#define CR(x) const x&
#define trav(it, v) for(typeof((v).begin()) it = (v).begin(); it!=(v).end(); ++it)
#define rep(i,a,b) for(int i=a;i<b;++i)
#define pb push_back
#define PT(x) (cout<<x<<endl)
using namespace std;
typedef long long ll;
typedef unsigned long long ull;
typedef double fl;
typedef vector<int> vi;
typedef vector<vi> vvi;
typedef vector<fl> vf;
typedef vector<vf> vvf;
ll pascal[512][512];
vi bb(CR(string) s) { istringstream i(s);	int x;vi rt;while(i>>x) rt.pb(x);return rt; }
template<typename t>
ostream& operator<<(ostream& os, CR(vector<t>) v) { os << '['; 	rep(i,0,v.size())		if(i!=v.size()-1) os << v[i] << ' ';		else os << v[i] << ']';	return os; }
int PC(int n, int k) { if(0 <= k && k <= n) return pascal[n][k]; else cout << "fail!" << endl; return 0;}
int fac(int x) { return x>0?x*fac(x-1):1;}
int pos(int x) {return x>0?x:-x;}
int neg(int x) {return -pos(x);}

void init() {
	//compute binomial coefficients
	rep(n,0,100) rep(k,0,n+1) 
	if(k == n || k == 0) pascal[n][k] = 1; else pascal[n][k] = pascal[n-1][k-1]+pascal[n-1][k];
}
fl eps = 1e-9;
fl abs_(fl x) { return x > 0 ? x : -x; }
bool is_zero(fl x) { return abs_(x) < eps; }

void rref(vvf&M) {
	int m,n;//means matrix M has dimensions m x n
	m = M.size(); n = M[0].size();
	int I = 0, J = 0; // position of the entry to be made to 1
	cout << setprecision(3);
	while(I < m && J < n) {
		int i = I;
		while(i < m && is_zero(M[i][J])) { M[i][J] = 0.0f;  ++i; }
		if(i == m) {
			//cout << "col " << J << " is zero" << endl;
			++J;
			continue;
		}
		//cout << "(" << i << ", " << J << ") is non-zero" << endl;

		//swap rows i and I
		rep(j,J,n)
			swap(M[i][j], M[I][j]);

		//make (I,J) entry 1
		fl z = 1.0f/M[I][J];
		rep(j,J,n)
			M[I][j] *= z;
		//make (i, J) entries 0 for i != I 
		rep(i,0,m) if(i!=I) {
			fl z = M[i][J];
			rep(k,J,n)
				M[i][k] -= z*M[I][k];
			M[i][J] = 0.0f;
		}
		++I;
	}
}

void print_(vvf&M) {
	rep(i,0,M.size()) {
		rep(j,0,M[i].size()) {
			cout << M[i][j] << setw(4) << right;
		}
		cout << endl;
	}
}

int main(int argc,char**argv) {
	init();srand(time(0));
	int n, r;
	string s="pts1.txt";
	/* pts1.txt could look like this:
7 4

1 1 0 1 1 0 0
0 1 1 1 0 1 1
1 1 0 1 1 1 1
0 0 0 1 1 0 1
	*/
	if(argc > 1) s = argv[1];
	ifstream file(s.c_str());
	file >> n >> r;
	vvf M(r, vf(n+1));
	rep(i,0,r) {
		rep(j,0,n) {
			fl x;
			file >> x;
			M[i][j] = x;
		}
		M[i][n] = 1;
	}
	rref(M);
	print_(M);
	vi forced(n+1,false);
	rep(i,0,r) {
		int j = 0;
		while(j < (n+1) && is_zero(M[i][j])) ++j;
		if(j != n+1) forced[j] = 1;
	}
	int n_free = 0; 
	rep(i,0,n+1) {
		if(forced[i]) cout << "c_"<<i<<" is forced"<<endl;
		else ++n_free;
	}
	vvf eqs;
	for(int T = 0; T < n_free; ++T) {
	vf c(n+1);
	int k = 0; 
	rep(i,0,n+1) if(!forced[i]) { if(k ==T) c[i] = 1; else c[i] = 0; ++k; }
	for(int i = r-1; i >= 0; --i) {
		int j = 0;
		while(j < (n+1) && is_zero(M[i][j])) ++j;
		if(j != n+1) {c[j] = 0; int jj=j; while(jj < n+1) {c[j] -= c[jj]*M[i][jj]; ++jj;} }
	}
	eqs.push_back(c);
	}

	cout << "all binary vectors in that affine span:"<<endl;
	rep(t,0,1<<n) {
		bool sat = true;
		rep(u,0,eqs.size()) {
			fl sum = 0;
			rep(i,0,n) {
				if(t & (1<<i)) sum += eqs[u][i];
			}
			if(!is_zero(sum + eqs[u][n])) {sat=false;break;}
		}
		if(sat) {
			rep(i,0,n) if(t&(1<<i))cout<<"1";else cout<<"0"; cout << endl;
		}
	}


	return 0;
}

